@@ -1,10 +1,11 @@
-from typing import Any, List, TYPE_CHECKING, Tuple, Type, Union
+from typing import Any, List, Mapping, TYPE_CHECKING, Tuple, Type, Union
 
 import databases
 import sqlalchemy
 
 import ormar  # noqa I100
 from ormar import MultipleMatches, NoMatch
+from ormar.queryset import FilterQuery
 from ormar.queryset.clause import QueryClause
 from ormar.queryset.query import Query
 
@@ -13,16 +14,18 @@ if TYPE_CHECKING:  # pragma no cover
 
 
 class QuerySet:
-    def __init__(
+    def __init__(  # noqa CFQ002
         self,
         model_cls: Type["Model"] = None,
         filter_clauses: List = None,
+        exclude_clauses: List = None,
         select_related: List = None,
         limit_count: int = None,
         offset: int = None,
     ) -> None:
         self.model_cls = model_cls
         self.filter_clauses = [] if filter_clauses is None else filter_clauses
+        self.exclude_clauses = [] if exclude_clauses is None else exclude_clauses
         self._select_related = [] if select_related is None else select_related
         self.limit_count = limit_count
         self.query_offset = offset
@@ -31,6 +34,36 @@ class QuerySet:
     def __get__(self, instance: "QuerySet", owner: Type["Model"]) -> "QuerySet":
         return self.__class__(model_cls=owner)
 
+    def _process_query_result_rows(self, rows: List[Mapping]) -> List["Model"]:
+        result_rows = [
+            self.model_cls.from_row(row, select_related=self._select_related)
+            for row in rows
+        ]
+        rows = self.model_cls.merge_instances_list(result_rows)
+        return rows
+
+    def _populate_default_values(self, new_kwargs: dict) -> dict:
+        for field_name, field in self.model_cls.Meta.model_fields.items():
+            if field_name not in new_kwargs and field.has_default():
+                new_kwargs[field_name] = field.get_default()
+        return new_kwargs
+
+    def _remove_pk_from_kwargs(self, new_kwargs: dict) -> dict:
+        pkname = self.model_cls.Meta.pkname
+        pk = self.model_cls.Meta.model_fields[pkname]
+        if new_kwargs.get(pkname, ormar.Undefined) is None and (
+            pk.nullable or pk.autoincrement
+        ):
+            del new_kwargs[pkname]
+        return new_kwargs
+
+    @staticmethod
+    def check_single_result_rows_count(rows: List["Model"]) -> None:
+        if not rows:
+            raise NoMatch()
+        if len(rows) > 1:
+            raise MultipleMatches()
+
     @property
     def database(self) -> databases.Database:
         return self.model_cls.Meta.database
@@ -44,6 +77,7 @@ class QuerySet:
             model_cls=self.model_cls,
             select_related=self._select_related,
             filter_clauses=self.filter_clauses,
+            exclude_clauses=self.exclude_clauses,
             offset=self.query_offset,
             limit_count=self.limit_count,
         )
@@ -51,22 +85,32 @@ class QuerySet:
         # print(exp.compile(compile_kwargs={"literal_binds": True}))
         return exp
 
-    def filter(self, **kwargs: Any) -> "QuerySet":  # noqa: A003
+    def filter(self, _exclude: bool = False, **kwargs: Any) -> "QuerySet":  # noqa: A003
         qryclause = QueryClause(
             model_cls=self.model_cls,
             select_related=self._select_related,
             filter_clauses=self.filter_clauses,
         )
         filter_clauses, select_related = qryclause.filter(**kwargs)
+        if _exclude:
+            exclude_clauses = filter_clauses
+            filter_clauses = self.filter_clauses
+        else:
+            exclude_clauses = self.exclude_clauses
+            filter_clauses = filter_clauses
 
         return self.__class__(
             model_cls=self.model_cls,
             filter_clauses=filter_clauses,
+            exclude_clauses=exclude_clauses,
             select_related=select_related,
             limit_count=self.limit_count,
             offset=self.query_offset,
         )
 
+    def exclude(self, **kwargs: Any) -> "QuerySet":  # noqa: A003
+        return self.filter(_exclude=True, **kwargs)
+
     def select_related(self, related: Union[List, Tuple, str]) -> "QuerySet":
         if not isinstance(related, (list, tuple)):
             related = [related]
@@ -75,6 +119,7 @@ class QuerySet:
         return self.__class__(
             model_cls=self.model_cls,
             filter_clauses=self.filter_clauses,
+            exclude_clauses=self.exclude_clauses,
             select_related=related,
             limit_count=self.limit_count,
             offset=self.query_offset,
@@ -83,32 +128,26 @@ class QuerySet:
     async def exists(self) -> bool:
         expr = self.build_select_expression()
         expr = sqlalchemy.exists(expr).select()
-        # print(expr.compile(compile_kwargs={"literal_binds": True}))
         return await self.database.fetch_val(expr)
 
     async def count(self) -> int:
         expr = self.build_select_expression().alias("subquery_for_count")
         expr = sqlalchemy.func.count().select().select_from(expr)
-        # print(expr.compile(compile_kwargs={"literal_binds": True}))
         return await self.database.fetch_val(expr)
 
     async def delete(self, **kwargs: Any) -> int:
         if kwargs:
             return await self.filter(**kwargs).delete()
-        qry = Query(
-            model_cls=self.model_cls,
-            select_related=self._select_related,
-            filter_clauses=self.filter_clauses,
-            offset=self.query_offset,
-            limit_count=self.limit_count,
+        expr = FilterQuery(filter_clauses=self.filter_clauses,).apply(
+            self.table.delete()
         )
-        expr = qry.filter(self.table.delete())
         return await self.database.execute(expr)
 
     def limit(self, limit_count: int) -> "QuerySet":
         return self.__class__(
             model_cls=self.model_cls,
             filter_clauses=self.filter_clauses,
+            exclude_clauses=self.exclude_clauses,
             select_related=self._select_related,
             limit_count=limit_count,
             offset=self.query_offset,
@@ -118,6 +157,7 @@ class QuerySet:
         return self.__class__(
             model_cls=self.model_cls,
             filter_clauses=self.filter_clauses,
+            exclude_clauses=self.exclude_clauses,
             select_related=self._select_related,
             limit_count=self.limit_count,
             offset=offset,
@@ -128,30 +168,20 @@ class QuerySet:
             return await self.filter(**kwargs).first()
 
         rows = await self.limit(1).all()
-        if rows:
-            return rows[0]
+        self.check_single_result_rows_count(rows)
+        return rows[0]
 
     async def get(self, **kwargs: Any) -> "Model":
         if kwargs:
             return await self.filter(**kwargs).get()
 
+        expr = self.build_select_expression()
         if not self.filter_clauses:
-            expr = self.build_select_expression().limit(2)
-        else:
-            expr = self.build_select_expression()
+            expr = expr.limit(2)
 
         rows = await self.database.fetch_all(expr)
-
-        result_rows = [
-            self.model_cls.from_row(row, select_related=self._select_related)
-            for row in rows
-        ]
-        rows = self.model_cls.merge_instances_list(result_rows)
-
-        if not rows:
-            raise NoMatch()
-        if len(rows) > 1:
-            raise MultipleMatches()
+        rows = self._process_query_result_rows(rows)
+        self.check_single_result_rows_count(rows)
         return rows[0]
 
     async def all(self, **kwargs: Any) -> List["Model"]:  # noqa: A003
@@ -159,36 +189,27 @@ class QuerySet:
             return await self.filter(**kwargs).all()
 
         expr = self.build_select_expression()
-        # breakpoint()
         rows = await self.database.fetch_all(expr)
-        result_rows = [
-            self.model_cls.from_row(row, select_related=self._select_related)
-            for row in rows
-        ]
-        result_rows = self.model_cls.merge_instances_list(result_rows)
+        result_rows = self._process_query_result_rows(rows)
 
         return result_rows
 
     async def create(self, **kwargs: Any) -> "Model":
 
         new_kwargs = dict(**kwargs)
-
-        # Remove primary key when None to prevent not null constraint in postgresql.
-        pkname = self.model_cls.Meta.pkname
-        pk = self.model_cls.Meta.model_fields[pkname]
-        if (
-            pkname in new_kwargs
-            and new_kwargs.get(pkname) is None
-            and (pk.nullable or pk.autoincrement)
-        ):
-            del new_kwargs[pkname]
-
+        new_kwargs = self._remove_pk_from_kwargs(new_kwargs)
         new_kwargs = self.model_cls.substitute_models_with_pks(new_kwargs)
+        new_kwargs = self._populate_default_values(new_kwargs)
+
         expr = self.table.insert()
         expr = expr.values(**new_kwargs)
 
         # Execute the insert, and return a new model instance.
         instance = self.model_cls(**kwargs)
         pk = await self.database.execute(expr)
-        setattr(instance, self.model_cls.Meta.pkname, pk)
+        pk_name = self.model_cls.Meta.pkname
+        if pk_name not in kwargs and pk_name in new_kwargs:
+            instance.pk = new_kwargs[self.model_cls.Meta.pkname]
+        if pk and isinstance(pk, self.model_cls.pk_type()):
+            setattr(instance, self.model_cls.Meta.pkname, pk)
         return instance
